// A N N I T
// ****************************************************************************
// Artificial
// Neural
// Network
// Inversion
// Tool
// ****************************************************************************
// April 19, 2009  Co.Bohuslav Ruzek
//
// This source code is organized into several sections according the functionality
// of distinct procedures:
// **********************************************
// 1. INPUT FUNCTIONS
// 2. OUTPUT FUNCTIONS 
// 3. GRAPHICAL MONITORING OF THE INVERSE
// 4. INVERSION ENGINE
// **********************************************
// Ver. 1.3
// July 30 2009
//  
//
// Fixed the bug for the case of generating the population in 1D model space
// see lines 500 1nd 1131 of the previous version 1.1.
// Switching between inv() and pinv() call depending on the singularity of RBFN.H
// see line 614
//
// ---------------------

// S E C T I O N    F O R    I N P U T S

//-----------------------------------------------
function read_param(in_name)

  global ParDef;
  global VER;

  ParDef = [];
  VER = '1.3';
  ParDef.err = 0;
  ParDef.pfile = in_name;
  open_log('ANNI.log');
  [fd_in,err] = mopen(pwd()+'/'+in_name,'r');
  if(err)
    error('Parameter file not accessible');
    if(fd_log > 0)
      mfprintf(fd_log, 'Parameter file %s not accessible\n',pwd()+'/'+in_name);
      ParDef.err = ParDef.err + 64;
    return;
    end
  end
  inp_txt = mgetl(fd_in);
  i = 0;
  j = 1;
  n = size(inp_txt,1);
  k = 1;
  while(i < n & j <= 11) do
    i = i + 1;
    if(length(inp_txt(i)) == 0) continue; end;
    lch = part(inp_txt(i),1);
    if(lch == '#' | lch == '%') continue; end;
    select j
    case 1 then
      ParDef.Title = inp_txt(i);
      j = j + 1;
    case 2 then
      [ns,ParDef.n] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;
    case 3 then
      [ns,ParDef.m] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;
    case 4 then
      [ns,ParDef.q] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;
    case 5 then
      [ns,ParDef.max_iter] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;
    case 6 then
      [ns,ParDef.max_time] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;
    case 7 then
      [ns,ParDef.max_nFwd] = msscanf(1, inp_txt(i), '%d');
      if(ns == 1) then j = j + 1; end;      
    case 8 then
      [ns,ParDef.Chi] = msscanf(1, inp_txt(i), '%f');
      if(ns == 1) then j = j + 1; end;      
    case 9 then
      [ns,ParDef.fcnf, ParDef.fcn] = msscanf(1, inp_txt(i), '%s %s');
      if(ns == 2) then j = j + 1; end;      
    case 10 then
      [ns,ParDef.name(k),ParDef.min(k),ParDef.max(k),ParDef.ini(k),pflg] = msscanf(1, inp_txt(i), '%s %f %f %f %s');
      if(ns >= 3) then 
        ParDef.fix(k) = 0;
        if(ns == 3) then
          ParDef.ini(k) = ParDef.min(k) + (ParDef.max(k)-ParDef.min(k))*rand();
        end
        if(ns == 5) then
          if(pflg == '*')
            ParDef.fix(k) = 1;
          end
        end
      end;
      k = k + 1;
      if(k > ParDef.m) then j = j + 1; end;
    case 11 then
      [ns,ParDef.msol,ParDef.rsd] = msscanf(1, inp_txt(i), '%d %f');
      if(ns == 0) then ParDef.msol = 1; ParDef.rsd = 0.0; j = j + 1; end;      
      if(ns == 1) then ParDef.rsd = 1.e-3; j = j + 1; end;      
      if(ns == 2) then j = j + 1; end;      
    else
      break;
    end
  end
  if(j <> 12)
    ParDef.err = ParDef.err + 1;
  end
  mclose(fd_in);
  head_log();
  if(check_param())
    close_log();
  end  
  return

endfunction

//-----------------------------------------------
function [err] = check_param()

  global fd_log;
  global ParDef;

  if(ParDef.err <> 1 & fd_log > 0)
    if(ParDef.m < 1)
      mfprintf(fd_log, 'Number of parameters = %d (must be > 0)\n', ParDef.m);
      ParDef.err = ParDef.err + 2;
    end
    if(ParDef.n < 1)
      mfprintf(fd_log, 'Number of equations = %d (must be > 0)\n', ParDef.n);
      ParDef.err = ParDef.err + 4;
    end
    if(ParDef.q < ParDef.m)
      mfprintf(fd_log, 'Population size = %d (must be > %d)\n', ParDef.q, ParDef.m);
      ParDef.err = ParDef.err + 8;
    end
    if(size(ParDef.min,1) <> ParDef.m | size(ParDef.max,1) <> ParDef.m)
      mfprintf(fd_log, 'Inconsistent number of defined parameters %d - %d (must be %d)\n', ...
                        size(ParDef.min,1), size(ParDef.max,1), ParDef.m);
      ParDef.err = ParDef.err + 16; 
    end
    for i=1:ParDef.m
      if(ParDef.max(i) < ParDef.min(i))
        ParDef.err = ParDef.err + 32;
        mfprintf(fd_log, 'Bad range for parameter %d: <%f %f>\n', i, ParDef.min(i), ParDef.max(i));
        break;
      end
    end
  end
  ParDef.cen = (ParDef.max + ParDef.min)/2;
  ParDef.rng = (ParDef.max - ParDef.min)/2;
  err = ParDef.err;
  return;

endfunction


function outpar = StretchPar(inpar)

  if(ParDef.nf == 0)
    outpar = inpar;
    return;
  end
  outpar = ParDef.mm*[inpar; ParDef.inf];
        
endfunction


function ShrinkPar(mod)

  global ParDef;
  
  ParDef.nf = sum(ParDef.fix);
  ParDef.M  = ParDef.m;
  if(ParDef.nf == 0)
    return;
  end
  if(mod == 1)
    ParDef.mm = zeros(ParDef.m,ParDef.m);
    ir = 0;
    is = ParDef.m + 1;
    for ii=1:ParDef.m
      if(ParDef.fix(ii) == 0)
        ir = ir + 1;
        ParDef.mm(ii,ir) = 1;
      else
        is = is - 1;
        ParDef.mm(ii,is) = 1;
      end
    end

    ParDef.M   = ir;
    ParDef.ini = ParDef.mm'*ParDef.ini;
    ParDef.min = ParDef.mm'*ParDef.min;
    ParDef.max = ParDef.mm'*ParDef.max;
    ParDef.cen = ParDef.mm'*ParDef.cen;
    ParDef.rng = ParDef.mm'*ParDef.rng;

    ParDef.inf = ParDef.ini(ParDef.M+1:ParDef.m,:);
    ParDef.mif = ParDef.min(ParDef.M+1:ParDef.m,:);
    ParDef.maf = ParDef.max(ParDef.M+1:ParDef.m,:);
    ParDef.cef = ParDef.cen(ParDef.M+1:ParDef.m,:);
    ParDef.rnf = ParDef.rng(ParDef.M+1:ParDef.m,:);

    ParDef.ini = ParDef.ini(1:ParDef.M,:);
    ParDef.min = ParDef.min(1:ParDef.M,:);
    ParDef.max = ParDef.max(1:ParDef.M,:);
    ParDef.cen = ParDef.cen(1:ParDef.M,:);
    ParDef.rng = ParDef.rng(1:ParDef.M,:);
  end
  if(mod == -1)
    ParDef.ini = ParDef.mm*[ParDef.ini;ParDef.inf];
    ParDef.min = ParDef.mm*[ParDef.min;ParDef.mif];
    ParDef.max = ParDef.mm*[ParDef.max;ParDef.maf];
    ParDef.cen = ParDef.mm*[ParDef.cen;ParDef.cef];
    ParDef.rng = ParDef.mm*[ParDef.rng;ParDef.rnf];
  end
  return;

endfunction


// S E C T I O N    F O R    O U T P U T S


//-----------------------------------------------
function fd_log = open_log(name)

  global VER;
  global fd_log;
  global RBFN;

  t = getdate();
  RBFN.t_i = 0;
  RBFN.t_e = 0;
  tic();
  [fd_log, err] = mopen(pwd() + '/' + name,'at');
  if(err)
    error('Error in opening the protocol file');
  else
//    rand('seed', 31415926);
//    mfprintf(fd_log, 'ANNI ver.%s inversion (rand seed = %d)\n', VER, rand('seed'));
    mfprintf(fd_log, 'Started  %04d/%02d/%02d %02d:%02d:%02d.%03d\n',t(1),t(2),t(6), t(7), t(8), t(9), t(10));
  end
  return

endfunction

//-----------------------------------------------
function close_log();

  global fd_log;
  global ParDef Popul RBFN;

  if(fd_log > 0)
    t = getdate();
    RBFN.t_i = RBFN.t_i + toc();
    mfprintf(fd_log, 'Total inversion time %.3f s (%.3f internal %.3f external)\n', RBFN.t_i + RBFN.t_e, RBFN.t_i, RBFN.t_e);
    mfprintf(fd_log, 'Finished %04d/%02d/%02d %02d:%02d:%02d.%03d\n',t(1),t(2),t(6), t(7), t(8), t(9), t(10));
    mfprintf(fd_log, 'Error flag in the end %d\n',ParDef.err);
    if(ParDef.err ~= 0)
      mfprintf(fd_log, 'Bit encoding of potential errors:\n');
      mfprintf(fd_log, '1,2,4,8,16,32 ... Bad formatting/values in the instruction file\n');
      mfprintf(fd_log, '128 ... Inapropriate dimension of returned data vector\n');
      mfprintf(fd_log, '256 ... NaN inside user response to trial model\n');
      mfprintf(fd_log, '512 ... Inf inside user response to trial model\n');
    end
    mfprintf(fd_log, 'Solution Nr.%d (%d)\n', Popul.sol, Popul.nsl(Popul.sol));
    mfprintf(fd_log, '***************************\n');
    outpar = StretchPar(Popul.Par(:,Popul.ind(1)));
    for i=1:ParDef.m
      if(ParDef.fix(i) == 1)
        mfprintf(fd_log, '%5d *%9s %10.3e\n', i, ParDef.name(i), outpar(i));
      else
        mfprintf(fd_log, '%5d %10s %10.3e\n', i, ParDef.name(i), outpar(i));
      end      
    end
    mfprintf(fd_log, '***************************\n');
    mfprintf(fd_log, 'Norm = %10.3e\n', TrueFit(Popul.ind(1)));
    mfprintf(fd_log, '***************************\n');
    mfprintf(fd_log, 'Inversion statistics\n');
    nlg = max((RBFN.g+RBFN.l+RBFN.n+RBFN.k)/100, 1/100);
    mfprintf(fd_log, 'Number of improvements achieved by random            %d (%4.1f%%)\n', RBFN.g, RBFN.g/nlg);
    mfprintf(fd_log, 'Number of improvements achieved by linear regression %d (%4.1f%%)\n', RBFN.l, RBFN.l/nlg);
    mfprintf(fd_log, 'Number of improvements achieved by linear prediction %d (%4.1f%%)\n', RBFN.k, RBFN.k/nlg);
    mfprintf(fd_log, 'Number of improvements achieved by RBFN prediction   %d (%4.1f%%)\n', RBFN.n, RBFN.n/nlg);
    mfprintf(fd_log, 'Total number of function evaluations %d Total number of models used %d Saving ratio %4.1f%%\n', nFwd, Popul.nfw, (Popul.nfw-nFwd)/Popul.nfw*100);
    mfprintf(fd_log, '### End of inversion ###\n\n');
    mclose(fd_log);
    fd_log = -1;
  end
  return;

endfunction

//-----------------------------------------------
function head_log()

  global fd_log;
  global ParDef;

  if(fd_log > 0)
    mfprintf(fd_log, 'Processing the instruction file %s\n', ParDef.pfile);
    mfprintf(fd_log, '%s\n', ParDef.Title);
    mfprintf(fd_log, 'Data space dimension  %d\n', ParDef.n);
    mfprintf(fd_log, 'Model space dimension %d\n', ParDef.m);
    mfprintf(fd_log, 'Population size       %d\n', ParDef.q);
    mfprintf(fd_log, 'Number of solutions   %d\n', ParDef.msol);
    mfprintf(fd_log, 'Definition of parameters\n');
    mfprintf(fd_log, '   Nr       name        min        max\n');
    mfprintf(fd_log, '-----+---------+----------+-----------\n');
    for i=1:ParDef.m
      if(ParDef.fix(i) == 1)
        mfprintf(fd_log, '%5d %10s %10.3e (fixed)\n', i, ParDef.name(i), ParDef.ini(i));
      else
        mfprintf(fd_log, '%5d %10s %10.3e %10.3e\n', i, ParDef.name(i), ParDef.min(i), ParDef.max(i));
      end        
    end
    mfprintf(fd_log, '-----+---------+----------+-----------\n');
    mfprintf(fd_log, 'Maximum number of iter: %d\n', ParDef.max_iter);
    mfprintf(fd_log, 'Maximum inversion time: %d\n', ParDef.max_time);
    mfprintf(fd_log, 'Maximum number of eval: %d\n', ParDef.max_nFwd);
    mfprintf(fd_log, 'Chi2 level to reach: %10.3e\n', ParDef.Chi);
    mfprintf(fd_log, 'Forward modelling function %s/%s\n', ParDef.fcnf, ParDef.fcn);
  end
  return;

endfunction

//-----------------------------------------------
function log_inv(mess)

  global fd_log;
  global ParDef Popul RBFN;
  global nFwd iLoop;

  if(mess == "")
    mfprintf(fd_log, 'M------R------r---qq--cycle-ncall------time------misfit\n');
   return;
  end
  itime = RBFN.t_i + RBFN.t_e;
  if(fd_log > 0)
    mfprintf(fd_log, '%20s  %5d %5d %9.3f %11.4e\n', mess, iLoop, nFwd, itime, TrueFit(Popul.ind(1)));
  end
  return;

endfunction

//-----------------------------------------------
function log_post()

  global fd_log;
  global Popul;

  if(Popul.sol == 1)
    return;
  end

  fd_log = mopen('ANNI.log', 'a');
  if(fd_log > 0)
    ltxt = '+--------+';
    for ii=1:Popul.sol
      ltxt = ltxt + '-----------+';
    end
    mfprintf(fd_log, 'Summary on multiple solutions\n');
    mfprintf(fd_log, '%s\n', ltxt);
    mfprintf(fd_log, '|        |');
    for ii=1:Popul.sol
      mfprintf(fd_log,'   %3d%3d  |', ii, Popul.nsl(ii));
    end
    mfprintf(fd_log, '\n%s', ltxt);
    is = 0;
    it = ParDef.nf + 1;
    for i=1:ParDef.m
      if(ParDef.fix(i) == 1)
        it = it - 1;
        mfprintf(fd_log,'\n*%9s', ParDef.name(i));
        for ii=1:Popul.sol
          mfprintf(fd_log, ' %11.4e', ParDef.inf(it));
        end
      else
        is = is + 1;
        mfprintf(fd_log,'\n%10s', ParDef.name(i));
        for ii=1:Popul.sol
          mfprintf(fd_log, ' %11.4e', Popul.PP(is,ii));
        end
      end        
    end
  
    mfprintf(fd_log, '\n%s', ltxt);
    mfprintf(fd_log, '\nTrue fit  ');
      for ii=1:Popul.sol
        mfprintf(fd_log, ' %11.4e', Popul.CHI(ii));
      end
    mfprintf(fd_log, '\n\n');

    mfprintf(fd_log, 'Number of really different solutions %d\n', Popul.msl);
    pp  = zeros(Popul.m,Popul.msl);
    nsl = zeros(Popul.msl,1);
    ltxt = '+--------+';
    for i=1:Popul.msl
      ltxt = ltxt + '-----------+';
      for ii=1:Popul.sol
        if(Popul.nsl(ii) == i)
          nsl(i) = nsl(i) + 1;
          pp(:,i) = pp(:,i) + Popul.PP(:,ii);
        end
      end
      if(nsl(i) > 0)
        pp(:,i) = pp(:,i)/nsl(i);
      end
    end

    mfprintf(fd_log, '%s\n', ltxt);
    mfprintf(fd_log, '|        |');
    for ii=1:Popul.msl
      mfprintf(fd_log,'   %6d  |', ii);
    end
    mfprintf(fd_log, '\n%s', ltxt);
    is = 0;
    it = ParDef.nf + 1;
    for i=1:ParDef.m
      if(ParDef.fix(i) == 1)
        it = it - 1;
        mfprintf(fd_log,'\n*%9s', ParDef.name(i));
        for ii=1:Popul.msl
          mfprintf(fd_log, ' %11.4e', ParDef.inf(it));
        end
      else
        is = is + 1;
        mfprintf(fd_log,'\n%10s', ParDef.name(i));
        for ii=1:Popul.msl
          mfprintf(fd_log, ' %11.4e', pp(is,ii));
        end
      end
    end
    mfprintf(fd_log, '\n%s', ltxt);
    mfprintf(fd_log, '\nNumber of roots');
      for ii=1:Popul.msl
        mfprintf(fd_log, '%4d        ', nsl(ii));
      end
    mfprintf(fd_log, '\n\n');
    mclose(fd_log);
    fd_log = -1;
  end
  return;

endfunction

// I N V E R S I O N    E N G I N E    S E C T I O N

//-----------------------------------------------
// How the structure 'Popul' is designed and used:
// -----------+----------------------------------
// Popul.q   ... total current size of the population
// Popul.qq  ... size of the subpopulation just used for approximation (Popul.qq <= Popul.q)
// Popul.qm  ... maximum allowed size of the population (Popul.qq <= Popul.qm)
// Popul.m   ... dimension of the model space (number of parameters)
// Popul.n   ... dimension of the data space (number of equations)
// Popul.nm  ... number of degree of freedom ( = n-m)
//
// Popul.nf  ... number of radial basis functions used
// Popul.np  ... number of patterns used for learning the RBFN
//
// Popul.Par() ... individuals in the population (model space)
// Popul.Dat() ... individuals in the population (data space)
// Popul.Cp()  ... scaling of individual parameters such that each margin of
//                 the parameter hyperrectangle measures 1
// Popul.Cl()  ...
//
// Popul.Born()... index specifying iteration cycle during which this individual has been created
// Popul.Stat()... character specifying method used for creation of this individual
//                 'I'=random generation during the initialization stage
//                 'N'=RBFN prediction corresponding to zero residuals
//                 'L'=supposing linear problem
//                 'G'=random generation of individuals forming the predicting body
//                 'K'=prediction by Kriging method to zero residuals (currently not used)
// Popul.Mask()... mask defining if the individual will (=1) or will not (=0) be used for approximation
// Popul.QQ()  ... sparse matrix corresponding to Popul.Mask used for fast selection of masked individuals
//                 multiplying by QQ from the left produces selection of the subpopulation from the whole population
//                 selected_par = Popul.par*Popul.QQ
//                 selected_dat = Popul.dat*Popul.QQ;
// Popul.map() ... indices of individuals in the population used for predicting RBFN 
//
// Popul.fcn   ... specification of the RBF interpolating function
//                 (1=Gaussian,2=Linear,3=Cauchy,-1=simple linear model)
// Popul.Chi() ... true chi-square of data
// Popul.chi() ... sorted list of Popul.Chi() in increasing order
// Popul.ind() ... list of Popul.Chi() indices to original positions
// Popul.nfw   ...
//
// RBFN.par()  ... selection of Popul.Par used for training the network
// RBFN.dat()  ... selection of Popul.Dat used for training the network
// RBFN.Cd     ... metric tensor Md = RBFN.Cd'*RBFN.Cd ensuring transformation of the RBFN.dat space such that the mean
//                 distance between all the selected vectors <|RBFN.dat(:,i)-RBFN.dat(:,j)|> = 1, i != j
// RBFN.n
// RBFN.l
// RBFN.k
// RBFN.g
//

function _init_()

  global Popul ParDef;
  global nFwd iLoop;
  global RBFN;
 
  RBFN.t_i = RBFN.t_i + toc();
  tic();
  execstr(msprintf('%s(0, StretchPar(ParDef.ini));', ParDef.fcn));
  RBFN.t_e = RBFN.t_e + toc();
  tic();
  nFwd       = 0;
  iLoop      = 0;
  Popul.sol  = 0;
  Popul.q    = 0;
  Popul.qm   = ParDef.M*100;
  Popul.m    = ParDef.M;
  Popul.n    = ParDef.n;
  Popul.nm   = max(Popul.n-Popul.m,1);
  Popul.nfw  = 0;
  
  Popul.qq   = ParDef.q;
  Popul.nf   = ParDef.q;
  Popul.np   = ParDef.q;
  Popul.fcn  = -1;
  Popul.rc   = sqrt(Popul.m)/2;
  Popul.Cl   = ParDef.rng;
  Popul.Cp   = Popul.Cl.^(-1);
  Popul.PP   = [];
  Popul.DD   = [];

  RBFN.dat = [];
  RBFN.par = [];
  RBFN.chi = [];
  RBFN.Cd  = [];
  RBFN.cen = [];
  RBFN.L   = ones(Popul.m,1);
  RBFN.R   = Popul.rc;
  RBFN.fcn = Popul.fcn;
  RBFN.n   = 0;
  RBFN.l   = 0;
  RBFN.k   = 0;
  RBFN.g   = 0;
  RBFN.r   = dist_s2();
  RBFN.b   = 1.5/2;
  
  Select(0, 'R', RBFN.R);
  RBFN.cen = ParDef.cen;
  i = 0;
  if(Select(ParDef.ini, 'I', 0) ~= 0)
      i = i + 1;
  end
  while(i ~= ParDef.q)
    xaux = rand(ParDef.M,1)-0.5;
    if(ParDef.M > 1)
      xmag = norm(xaux);
      if(xmag == 0)
        continue;
      end
      xaux = xaux/norm(xaux);
    end
    xaux = adjust(xaux.*Popul.Cl*RBFN.R+RBFN.cen,RBFN.cen);
    if(Select(xaux, 'I', 0) ~= 0)
      i = i + 1;
    end  
  end
  Select(0, 'M', RBFN.R);
  return;
    
endfunction


//-----------------------------------------------
function _reinit_()

  global Popul ParDef;
  global nFwd iLoop;
  global RBFN;
 
  close_log();
  open_log('ANNI.log');
  ShrinkPar(-1);
  head_log();
  ShrinkPar(+1);
  log_inv('');
  
  RBFN.dat = [];
  RBFN.par = [];
  RBFN.chi = [];
  RBFN.Cd  = [];
  RBFN.cen = ParDef.cen;
  RBFN.L   = ones(Popul.m,1);
  RBFN.R   = Popul.rc;
  RBFN.fcn = Popul.fcn;
  RBFN.n   = 0;
  RBFN.l   = 0;
  RBFN.k   = 0;
  RBFN.g   = 0;
  RBFN.r   = dist_s2();
  RBFN.b   = 1.5/2;  
  Popul.qq   = ParDef.q;
  Popul.nf   = ParDef.q;
  Popul.np   = ParDef.q;
  Popul.fcn  = -1;
  Popul.enc  = iLoop;
  Popul.nfw  = 0;
  nrm  = int(Popul.q/2);
  rind = sort(Popul.ind(1:nrm));
  for i=nrm:-1:1
      del_individual(rind(i));
  end
  nFwd     = 0;
  iLoop    = 0;

  return;
    
endfunction


//-----------------------------------------------
function _exit_()

  global Popul;
  global RBFN;
  
  close_log();
  log_post();
  clear ParDef;
  clear Popul;
  clear RBFN;
  return;

endfunction

//-----------------------------------------------
function [iB, fB] = EvalPop()
//
// The current population is classified according the chi^2 misfit. Since the size of the population
// is not fixed and may have changed during the computations, the current size has to be checked for and
// here is the only place where the adjustment is done.
//

  global Popul;

  AddPenalty();
  Popul.Chi = diag(Popul.Dat'*Popul.Dat)/Popul.nm + Popul.Pen;
  [Popul.chi,Popul.ind] = gsort(Popul.Chi, 'r', 'i');
  iB = Popul.ind(1);
  fB = Popul.chi(1);  
  return;

endfunction

//-----------------------------------------------
function p = AddConstraint(p)
//
//

  global Popul ParDef;
  global nFwd iLoop;

  if(Popul.sol > 0)
    w = 1;//max(1-iLoop/Popul.enc,0.00);
    if(w > 0)
      dn = zeros(1,Popul.sol);
      ri = dist_p2(p,Popul.PP(:,1:Popul.sol));
      dp = p*ones(1,Popul.sol) - Popul.PP(:,1:Popul.sol);
      for i=1:Popul.sol
        dn(i) = norm(dp(:,i));
    end
      dp   = dp./(ones(Popul.m,1)*dn);
      dpen = sum((ones(Popul.m,1)*(w*exp(-ri/ParDef.rsd))).*dp, 'c');
    p = p + dpen;
    end
  end
  return;

endfunction

//-----------------------------------------------
function AddPenalty()
//
//

  global Popul ParDef;
  global nFwd iLoop;

  Popul.q = size(Popul.Dat,2); 
  if(Popul.sol > 0)
    w = 1;//max(1-iLoop/Popul.enc, 0);
    if(w > 0)
      mpen  = w*mean(Popul.Chi-Popul.Pen);
      Popul.Pen = zeros(Popul.q,1);
        for i=1:Popul.sol
          psol = Popul.PP(:,i);
          ri = dist_p2(Popul.Par(:,1:Popul.q),psol);
          dpen = mpen*exp(-ri/ParDef.rsd);
          Popul.Pen = Popul.Pen + dpen;
        end
    end
  end
  return;

endfunction

//-----------------------------------------------
function chi = TrueFit(ind)
//
// "True fit" of the ind-th individual is computed. "True fit" does
// not include penalty term.
//

  global Popul;

  if(Popul.sol == 0)
    chi = Popul.Chi(ind);
  else
    chi = Popul.Chi(ind) - Popul.Pen(ind);
  end

  return;

endfunction

//-----------------------------------------------
function SetupRBFN()
//
// The design matrix H and the matrix of weights W are computed
// Popul.nf = number of radial basis functions
// Popul.np = number of learning patterns
//
  
  global Popul;
  global RBFN;
  
  RBFN.par = Popul.Par*Popul.QQ;
  RBFN.dat = Popul.Dat*Popul.QQ;
  RBFN.chi = (Popul.Chi'*Popul.QQ)';
  [chi,ind] = gsort(RBFN.chi, 'r', 'i');
  RBFN.ib = ind(1);
  RBFN.fb = chi(1);
  RBFN.iw = ind(size(ind,1));
  RBFN.fw = chi(size(chi,1));
  clear chi;
  clear ind;
// Section for linear predictor
  if(RBFN.fcn == -1) then
    meanP = mean(RBFN.par, 'c');
    meanD = mean(RBFN.dat, 'c');
    PC = RBFN.par;
    DC = RBFN.dat;
    PC = PC - meanP*ones(1,Popul.np);
    DC = DC - meanD*ones(1,Popul.np);
    RBFN.H = PC*pinv(DC);
    RBFN.p = meanP - RBFN.H*meanD;
    RBFN.W = [];
    return
  end
// ---------------------------------

  nd = Popul.np*(Popul.np-1)/2;
  dd = zeros(Popul.n,nd);

  nk = Popul.np-1;
  nl = 1;
  for i=1:Popul.np-1
    dd(:,nl:nl+nk-1)= RBFN.dat(:,i)*ones(1, nk)-RBFN.dat(:,i+1:Popul.np);
    nl = nl + nk;
    nk = nk - 1;
  end

  dd = (dd*dd')*nd/Popul.n;
  [U,S,V] = svd(dd);
  S = diag(S);
  stol = S(1)*1.e-6; // %eps;
  ind1= find(sign(S-stol)+1);
  ind0= find(sign(S-stol)-1);
  S(ind1) = S(ind1).^(-1);
  S(ind0) = S(ind0)*0;      
//  RBFN.Cd = V*diag(S)*U';
  RBFN.Cd = diag(sqrt(S))*V';

//  if(rcond(dd) < 1.e-6)
//    RBFN.Cd = inv(dd);
//  else
//    RBFN.Cd = inv(dd);    
//  end
  clear dd;

// Section for Kriging predictor (currently in development)
  if(RBFN.fcn == -2) then    
    RBFN.H = zeros(Popul.np+1, Popul.np+1);
    RBFN.H(Popul.np+1,1:Popul.np) = ones(1,Popul.np);
    for i=1:Popul.np
      RBFN.H(i+1:Popul.np,i) = (r2_rbfn(i+1:Popul.np,RBFN.dat(:,i)))^RBFN.b;
    end
    RBFN.H = RBFN.H + RBFN.H';
  return;
  end
// ---------------------------------
  
// Section for RBFN predictor
  RBFN.H = eye(Popul.np, Popul.nf)*0.5;
  RBFN.W = zeros(Popul.nf, Popul.m);
  RBFN.P = zeros(Popul.np, Popul.np);
  Ip     = eye(Popul.np,Popul.np);

  select (RBFN.fcn)
    case 1 then  // Gaussian
      ;
    case 2 then  // pseudolinear
      ;
    case 3 then  // Cauchy
      ;
    case 4 then  // inverse multiquadric
      RBFN.Cd = RBFN.Cd*Popul.nf;
  end

  for i=1:Popul.np do
    RBFN.H(i,i+1:Popul.nf) = h(i+1:Popul.nf, RBFN.dat(:,i))';
  end
  RBFN.H = RBFN.H + RBFN.H';
  lamda = mean(RBFN.L);
  H2    = RBFN.H'*RBFN.H;
  [U,S,V] = svd(H2);
  S = diag(S);
  for i=1:Popul.m
    lam00 = 0.0;
    lnorm = 1.0;
    iter  = 0;
    W     = RBFN.W(:,i);
    Hpar  = RBFN.H'*RBFN.par(i,:)';
    while(iter < 1000 & lnorm > 0.01) do
      Sl  = S + lamda;
      stol= max(Sl)*1.e-6; // %eps;
      ind1= find(sign(Sl-stol)+1);
      ind0= find(sign(Sl-stol)-1);
      Sl(ind1) = Sl(ind1).^(-1);
      Sl(ind0) = Sl(ind0)*0;
      VAR = V*diag(Sl)*U';      
      RBFN.P = Ip - RBFN.H*VAR*RBFN.H';
      P2 = RBFN.P*RBFN.P';
      V2 = VAR*VAR;
      W  = VAR*Hpar;
      lamdn  = (W'*VAR*W*trace(RBFN.P));
      if(lamdn == 0) then
        break;
      end
      lamda  = RBFN.par(i,:)*P2*RBFN.par(i,:)'*trace(VAR-lamda*V2)/lamdn;
      if(lamda < 0 | lamda > S(1)) then
        break;
      end
      RBFN.W(:,i) = W;
      lnorm  = abs(lamda-lam00)/lamda;
      lam00  = lamda;
      iter = iter + 1;
    end
    RBFN.L(i)   = lamda;
  end
  return;

endfunction

//-----------------------------------------------
function sel_flg = Select(paux, id, r0)
//
//
//
  global Popul;

// Resetting the previous selection
  if(id == 'R')
    Popul.Mask = zeros(Popul.q,1);
    Popul.map  = [];
    Popul.QQ   = [];
    sel_flg    = 0;
    return;
  end

// Mapping of previously selected individuals
  if(id == 'M')
    ii = 0;
    sel_flg = sum(Popul.Mask);
    for i=1:Popul.q do
      if(Popul.Mask(i) == 0) then continue; end
      ii = ii + 1;
      Popul.map(ii) = i;
    end
    
    Popul.QQ = sparse([Popul.map,[1:Popul.qq]'],ones(Popul.qq,1),[Popul.q,Popul.qq]);
    return;
  end


// Trying to find individual within the current population sufficiently close to xaux
// and still not selected.
// This way saving the number of forward modelling could be achieved.
  if(Popul.q > Popul.m)
    dd = sqrt(dist_p2(Popul.Par(:,1:Popul.q),paux));
    dd = max(dd,Popul.Mask*max(dd));
    [rq, iq] = gsort(dd,'r','i');
    for i=1:Popul.q
      if(rq(i) < r0 & Popul.Mask(iq(i)) == 0) then
        Popul.Mask(iq(i)) = 1;
        sel_flg = iq(i);
        Popul.nfw = Popul.nfw + 1;
        return;
      end
      if(rq(i) > r0)
        break;
      end
    end
    clear dd;
  end
// No individual sufficiently close to the inserted one was found, so forward modelling will be necessary.  
  [i,f] = add_individual(%nan, paux, id, nFwd);
  if(Popul.Mask(i,1) == 0)
    Popul.Mask(i,1) = 1;
    Popul.nfw = Popul.nfw + 1;
    sel_flg = i;
  else
    sel_flg = 0;
  end
  return;

endfunction


//-----------------------------------------------
function SetupStrategy()

  global Popul;
  global RBFN;
  
  if(rand() < 0.5)
    qq = Popul.m+1 + int(rand()*(9*Popul.m-1));
  else
    qq = ParDef.q;
  end
  Popul.nf = qq;
  Popul.qq = qq;
  Popul.np = qq;

  if(rand() < 0.5)
    Popul.rc = sqrt(Popul.m)/2;
  else
    Popul.rc = 1/2;
  end  
  if(rand() < 0.5)
    RBFN.r = dist_s2();
  else
    RBFN.r = 0;
  end  

endfunction

//-----------------------------------------------
function rc = dist_s2()
  
  
    par = grand(Popul.m,Popul.qq,'nor',0,1); // Normal distribution should be here
    par = par./(ones(Popul.m,1)*(sqrt(diag(par'*par)')));
    dx = zeros(Popul.qq-1,1);
    for jj=1:Popul.qq-1
        dp  = par(:,jj)*ones(1,Popul.qq-jj)-par(:,jj+1:Popul.qq);
        dp = dp.^2;
        dx(jj) = min(sum(dp,'r'));
    end
    rc = sqrt(mean(dx));
endfunction

//-----------------------------------------------
function [d] = dist_p2(p1, p2)
//
// This is the analogon to dist_d2(). Here both p1 and p2 can be
// matrices:
//      d(i,j) = dist(p1(:,i) - p2(:,j))
//
        
  np1 = size(p1,2);
  np2 = size(p2,2);
  d   = zeros(np1, np2);
  
  p1  = p1.*(Popul.Cp*ones(1,np1));
  p2  = p2.*(Popul.Cp*ones(1,np2));
  if(np1 < np2)
    for i=1:np1
      pp = p1(:,i)*ones(1,np2) - p2;
      d(i,:) = sum(pp.*pp,'r');
    end
  else
    for i=1:np2
      pp = p2(:,i)*ones(1,np1) - p1;
      d(:,i) = sum(pp.*pp,'r')';
    end
  end
  clear p1 p2 pp;
  return;

endfunction

//-----------------------------------------------
function [r2] = r2_rbfn(i, d)
//
// Squared distance between the vector d and the centre of the i-th radial basis function.
// Since the i-th radial basis function is not centered in the i-th individual of the population,
// mapping is necessary. The RBFN.Cd matrix must be specified in advance. This matrix defines
// how to compute distances in data space. Metric tensor in data space Md is defined as
//                        Md = RBFN.Cd'*RBFN.Cd
// and distance is
//                      dist(r) = r'*Md*r
// which is identical to
//                      dist(r) = r'*RBFN.Cd'*RBFN.Cd*r = (RBFN.Cd*r)'*(RBFN.Cd*r)
// and the last one equation is coded here.

  global Popul;
  global RBFN;

  dd = d*ones(1,size(i,2)) - Popul.Dat(:,Popul.map(i));
  dd = RBFN.Cd*dd;
  r2 = sum(dd.*dd, 'r')';

  return;

endfunction


//-----------------------------------------------
function [f] = h(i, d)
//
// Core implementation of the i-th basis function.
// r2 ... squared distance of the i-th radial basis function center and the input vector d
// f  ... contribution of the i-th radial basis function to the interpolation.
//

  global Popul;

  r2 = r2_rbfn(i, d);

  select RBFN.fcn
  case 1 then
    f = exp(-r2/2);            // Gaussian
  case 2 then
    f = 1-sqrt(r2);            // piecewise "linear" 
  case 3 then
    f = exp(-sqrt(r2));        // Cauchy function
  case 4 then
    f = (sqrt(1+sqrt(r2)))^(-1);    // Inverse Multiquadric
  else
    f = 0.0;                   // unknown function
  end
  return;

endfunction


//-----------------------------------------------
function err = check_data(dd)
//
// It can happen that user defined function returns inappropriate
// result. Such cases are detected here.
//
  sd  = size(dd);
  if(min(sd) ~= 1 | max(sd) ~= Popul.n)
    ParDef.err = bitor(ParDef.err,128);
  end
  if(sum(isnan(dd)) > 0)
    ParDef.err = bitor(ParDef.err,256);
  end
  if(sum(isinf(dd)) > 0)
    ParDef.err = bitor(ParDef.err,512);
  end
  err = ParDef.err;
  return;
endfunction

//-----------------------------------------------
function ior = bitor(i1, i2)
//
  ior = 0;
  iex = 1;
  while(i1 | i2)
    if(modulo(i1,2) | modulo(i2,2))
      ior = ior + iex;
    end
    iex = iex * 2;
    i1 = int(i1/2);
    i2 = int(i2/2);
  end
  return;
endfunction

//-----------------------------------------------
function [r,p] = predict(d)
//
// Core function of this code. Interpolating RBFN must be already adjusted. Arbitrary input vector d is
// fed to the interpolation engine.
//

  global Popul;
  global RBFN;
      
  select RBFN.fcn
  case -1 then  // pure linear prediction
    p = RBFN.H*d + RBFN.p;
  case -2 then  // Kriging
    dd = [(r2_rbfn(1:Popul.nf, d))^RBFN.b; 1];
//    p = [RBFN.par zeros(Popul.m,1)]*(RBFN.H\dd);
    p = [RBFN.par zeros(Popul.m,1)]*(pinv(RBFN.H)*dd);
    clear dd;
  else          // RBFN prediction
    dd = zeros(Popul.nf,1);
    dd(1:Popul.nf) = h(1:Popul.nf,d);
    p  = RBFN.W'*dd;
    clear dd;
  end
  dc = dist_p2(RBFN.cen, RBFN.par(1:Popul.m,1:Popul.nf));
  p  = adjust(p, RBFN.cen);
  pr = mean(sqrt(dc));
  pp = sqrt(dist_p2(RBFN.cen, p));
  r  = pp/pr;
  
  clear dc;
  return;

endfunction

// ----------------------------------------------
function p = adjust(p,cen)
//
// This routine prevents the vector p to leave the allowed hypercube
//

  global ParDef;
  p = AddConstraint(p);
  for i=1:ParDef.M
    cp = cen - p;
    if(cp(i) == 0)
      continue;
    end
    if(p(i) < ParDef.min(i))
      p = p + cp*(ParDef.min(i)-p(i))/cp(i);
    elseif(p(i) > ParDef.max(i))
        p = p + cp*(ParDef.max(i)-p(i))/cp(i);
    end
  end
  return;

endfunction


// ----------------------------------------------
function [i,f] = add_individual(Di, Pi, s, n)
//
// New individual is conditionally appended at the end of the current population. The necessary
// condition for accepting this individual is that it must differ from all other still existing
// individuals within the population (duplicity of individuals is thus restricted).
// Before inserting the size of the current population is checked.
// 
//
  global Popul RBFN;
  global nFwd;

// If the inserted individual is identical with any of already present individuals in the population
// nothing happens

  if(Popul.q > 0)
      dp = Popul.Par - Pi*ones(1,Popul.q);
      dp = sum(dp.*dp,1);
      [fx, ix] = gsort(dp, 'c', 'i');
      if(fx(1) == 0)
          i = ix(1);
          f = Popul.Chi(i);
          return;
      end
  end

// If the inserted individual was not evaluated, it will be done just now
  if(isnan(Di(1)) == %T)
    RBFN.t_i = RBFN.t_i + toc();
    tic();
    execstr(msprintf('Di=%s(1, StretchPar(paux))', ParDef.fcn));
    RBFN.t_e = RBFN.t_e + toc();
    tic();
    if(check_data(Di))
      Di = 1.e+10*ones(Popul.n,1);
    end
    nFwd = nFwd + 1;
  end

// If the size of the population should exceed the allowed limit, neccesary number of individuals
// are removed according their Chi
  ndel = Popul.q - Popul.qm + 1;
  itst = Popul.q;
  while(ndel > 0 & itst > 1) then
    idel = Popul.ind(itst);
    if(Popul.Mask(idel) == 0) then
      del_individual(idel);
      ndel = ndel - 1;
      itst = itst - 1;
    end
    itst = itst - 1;
  end
// Finally the individual is inserted into the population  
  Popul.q = Popul.q + 1;
  Popul.Dat(:,Popul.q) = Di;
  Popul.Pen(Popul.q)   = 0;
  Popul.Par(:,Popul.q) = Pi;
  Popul.Born(Popul.q)  = n;
  Popul.Stat(Popul.q)  = s;
  Popul.Mask(Popul.q)  = 0;
  Popul.Chi(Popul.q)   = (Popul.Dat(:,Popul.q)'*Popul.Dat(:,Popul.q))/Popul.nm;
  [Popul.chi,Popul.ind] = gsort(Popul.Chi, 'r', 'i');
  i = Popul.q;
  f = Popul.Chi(i);
  return;

endfunction


// ----------------------------------------------
function del_individual(qx)
//
// Raw deletion of the qx-th individual from the population. After doing this, many things
// will not work. All necessary relations have to be set up again. (e.g. Popul.Mask(),Popul.QQ()).
//

  global Popul;

  if(qx < 1 | qx > Popul.q) return;
  end
  q1 = qx - 1;
  q2 = qx + 1;
  q3 = Popul.q;

  Popul.Dat  = [Popul.Dat(:,1:q1),Popul.Dat(:,q2:q3)];
  Popul.Pen  = [Popul.Pen(1:q1)',Popul.Pen(q2:q3)']';
  Popul.Par  = [Popul.Par(:,1:q1),Popul.Par(:,q2:q3)];
  Popul.Born = [Popul.Born(1:q1)',Popul.Born(q2:q3)']';
  Popul.Stat = [Popul.Stat(1:q1)',Popul.Stat(q2:q3)']';
  Popul.Mask = [Popul.Mask(1:q1)',Popul.Mask(q2:q3)']';
  Popul.Chi  = [Popul.Chi(1:q1)',Popul.Chi(q2:q3)']';
  Popul.q = Popul.q - 1;
  [Popul.chi,Popul.ind] = gsort(Popul.Chi, 'r', 'i');

  return

endfunction


//-----------------------------------------
function [ret_val] = stop_test()

  global fd_log;
  global Popul ParDef;
  global nFwd iLoop;

  iLoop   = iLoop + 1;
  itime   = toc();
  ret_val = 0;
  if(iLoop >= ParDef.max_iter & ParDef.max_iter > 0) // checking number od loops
    log_inv('');
    mfprintf(fd_log, 'STOP:::Maximum number of iterations reached\n');
    ret_val = 1;
  end
  if(itime > ParDef.max_time & ParDef.max_time > 0)  // checking elapsed time
    log_inv('');
    mfprintf(fd_log, 'STOP:::Computing time exceeded the limit\n');
    ret_val = 2;
  end
  if(nFwd > ParDef.max_nFwd & ParDef.max_nFwd > 0)   // checking number of function evaluations
    log_inv('');
    mfprintf(fd_log, 'STOP:::Maximum number of evaluations reached\n');
    ret_val = 3;
  end
  if(TrueFit(Popul.ind(1)) < ParDef.Chi)
    log_inv('');
    mfprintf(fd_log, 'STOP:::Final misfit below the significant level\n');
    ret_val = 4;  
  end  
  if(ParDef.err ~= 0)
    log_inv('');
    mfprintf(fd_log, 'STOP:::Run time error detected\n');
    ret_val = 5;
  end 
  return;

endfunction


//-----------------------------------------
function dsol = IndexSol()
//
// Popul.sol .... total number of roots found till now
// Popul.msl .... total number of different solutions found (is computed here)
// Popul.nsl[] .. numbers (indices) of all found solutions (may be duplicated)
//
// Output: dsol = 0 ... last solution is unique
//         dsol > 0 ... last solution duplicates some solution already found

  global Popul ParDef;
  
  Popul.msl    = 1;
  Popul.nsl(1) = 1;
  for i=2:Popul.sol
    dp2 = dist_p2(Popul.PP(:,i), Popul.PP(:,1:i-1));
    [dp2,dpn] = gsort(dp2,'c','i');
    if(dp2(1) < ParDef.rsd)
        Popul.nsl(i) = Popul.nsl(dpn(1));
    else
      Popul.msl = Popul.msl + 1;
      Popul.nsl(i) = Popul.msl;
    end
  end
  dsol = 0;
  for i=1:Popul.sol-1
    if(Popul.nsl(i) == Popul.nsl(Popul.sol))
      dsol = Popul.nsl(i);
      return;
    end
  end
  return;
  
endfunction



// -----------------------------------------------------------------------------

function [x,fval] = ANNI(Fname);

  global Popul ParDef;
  global nFwd iLoop;
  global RBFN;
  global fd_log;
//  global xs;

  Popul  = [];
  ParDef = [];
  RBFN   = [];
  x      = [];
  fval   = [];
  read_param(Fname);
  if(~ParDef.err) then
    exec(ParDef.fcnf);
    ShrinkPar(1);
    _init_();
    [ib, fb] = EvalPop();
    gb = fb;
//    mprintf('Best CHI^2 = %10.3e individual # %3.3d fcn = %d %s %5.5d\n', fb, ib, RBFN.fcn, Popul.Stat(Popul.ind(1)), nFwd);
    log_inv('');
    d0  = zeros(ParDef.n,1);
    nwait = 0;
    nstop = 0;
// M A I N    I T E R A T I O N    C Y C L E
    while(nstop == 0) do
      nstop = stop_test();
      if(nstop ~= 0)
        RBFN.t_i = RBFN.t_i + toc();
        tic();
        execstr(msprintf('%s(3, StretchPar(Popul.Par(:,Popul.ind(1))))', ParDef.fcn));
        RBFN.t_e = RBFN.t_e + toc();
        tic();
        Popul.sol = Popul.sol + 1;
        Popul.PP(:,Popul.sol) = Popul.Par(:,Popul.ind(1));
        Popul.DD(:,Popul.sol) = Popul.Dat(:,Popul.ind(1));
        Popul.CHI(Popul.sol)  = TrueFit(Popul.ind(1));
        dsol = IndexSol();  
        if((Popul.msl < ParDef.msol) & (Popul.sol < ParDef.msol*2))
          _reinit_();
          if(dsol ~= 0)
            Popul.enc = Popul.enc*2;
//mprintf('Duplicated estimated convergency rate %d\n',Popul.enc);
          end
          gb = TrueFit(Popul.ind(Popul.q));
          nstop = 0;
        else
          continue;
        end
      end

// S E L E C T I O N    B L O C K      
      Select(0, 'R', RBFN.R);
      Select(RBFN.cen, 'G', 0);
      ns  = Popul.qq - 1;
      rc2 = (RBFN.R/10)^2;
      i   = 0;
      nxx = 0;
      nxc = ns*10;
      while(i ~= ns)
        nxx = nxx + 1;
        if(nxx == nxc)
          break;
        end
        xpar = rand(ParDef.M,1)-0.5;
        if(ParDef.M > 1)
          xmag = norm(xpar);
          if(xmag == 0)
            continue;
          end
          xpar = xpar/norm(xpar);
        end
        xpar = Popul.Cl*RBFN.R.*xpar+RBFN.cen;
        xpar = adjust(xpar,RBFN.cen);
        if(dist_p2(RBFN.cen, xpar) < rc2)
          continue;
        end
        if(Select(xpar, 'G', RBFN.R*RBFN.r) > 0)
          i = i + 1;
        end
      end
      if(nxx == nxc)
        RBFN.cen = ParDef.min + rand(ParDef.M,1).*(ParDef.max-ParDef.min);
        continue;
      end
      Select(0, 'M', RBFN.R);
// P R E D I C T I O N    B L O C K
      select(modulo(iLoop,3))
        case 0 then
          RBFN.fcn = -1;
          st = 'L';
        case 1 then
          RBFN.fcn = +4;
          st = 'N';
        case 2 then
          RBFN.fcn = -2;
          st = 'K';
      end
      SetupRBFN();    
      [r,p0] = predict(d0);
      RBFN.t_i = RBFN.t_i + toc();
      tic();
      execstr(msprintf('dd0=%s(1, StretchPar(p0))', ParDef.fcn));
      RBFN.t_e = RBFN.t_e + toc();
      tic();
      if(check_data(dd0) ~= 0)
        dd0 = 1.e+10*ones(Popul.n,1);
        continue;
      end
      nFwd = nFwd + 1;      
      [ip,fp] = add_individual(dd0,p0,st,nFwd);
      [ib,fb] = EvalPop();

// C O R R E C T I O N    B L O C K
      if(r < 1 & nwait) // predicted model is inside the volume spanned by predicting individuals
        RBFN.R = RBFN.R*(0.5+r/2);
        RBFN.R = max(RBFN.R, 1.e-3);
        RBFN.R = min(RBFN.R, Popul.rc);
        if(fp < RBFN.fb)
          st = st + 'A';
          RBFN.cen = p0;
          nwait = 0;
        else
          if(nwait >= Popul.m)
            nwait = 0;
            if(rand() < 0.5)
              st = st + 'F';
              RBFN.cen = ParDef.min + rand(ParDef.M,1).*(ParDef.max-ParDef.min);
            else
              st = st + 'H';
              RBFN.cen = Popul.Par(:,ib);
            end            
            RBFN.R = Popul.rc;
            SetupStrategy();
          else
            st = st + 'B';
            RBFN.cen = RBFN.par(:,RBFN.ib);
            nwait = nwait + 1;
          end    
        end
      else      // predicted model is outside the predicting body
        if(fp < RBFN.fb)
          st = st + 'C';
          RBFN.cen = p0;
          nwait = 0;
        else
          nwait = nwait + 1;
          if(nwait >= Popul.m)
            nwait = 0;
            if(rand() < 0.5)
              st = st + 'D';
              RBFN.cen = ParDef.min + rand(ParDef.M,1).*(ParDef.max-ParDef.min);
            else
              st = st + 'E';
              RBFN.cen = Popul.Par(:,ib);
            end            
            RBFN.R = Popul.rc;
            SetupStrategy();
          else
            st = st + 'X';
          end
        end
      end
            
//mprintf('R = %10.3f r=%10.3f minRchi = %10.3e pchi = %10.3e bestchi = %10.3e ', RBFN.R, r, min(RBFN.chi), fp, fb);      
//mprintf('Best CHI^2 = %10.3e individual # %4.4d fcn = %2d %s %s %5.5d %d\n', fb, ib, RBFN.fcn, Popul.Stat(Popul.ind(1)), st, nFwd, nwait);

// L O G G I N G    B L O C K
      if(fb < gb)
        select Popul.Stat(ib)
        case 'N' then
          RBFN.n = RBFN.n + 1;
        case 'L' then
          RBFN.l = RBFN.l + 1;
        case 'K' then
          RBFN.k = RBFN.k + 1;
        case 'G' then
          RBFN.g = RBFN.g + 1;
        end
        gb = fb;
      end
      log_inv(Popul.Stat(Popul.ind(1))+msprintf('%7.3f%7.3f%5d', RBFN.R, RBFN.r, Popul.qq));
    end    
  end

  if(ParDef.err == 0)
    x    = Popul.PP;
    fval = Popul.CHI;
    _exit_();
  end

endfunction


// -----------------------------------------------------------------------------

