// Example function to be solved by using the command
// ANNI('my_m.in');
//
// this file implements following system of equations:
// x1*x1 + 3*x1*x2 - x2 + 3 = 0
// 2*x1*x1 - x2 + 2*x2*x2 - 3 = 0
// -5*x1 + 2*x2 - 7 = 0
// which has unique solution x1 = -1, x2 = +1
// Obviously, we have three equations for two unknowns here.

function resid = my_fcn(fmode, param)

select fmode
    case 0
// Anytime ANNI has been started, this function is called with fmode = 0
// exactly once at the beginning of inversion. The purpose is to allow the user
// to make preparatory steps. Output can be arbitrary, since resid() is not used at all.
        mprintf('fcn0 call: Time for preparing your problem for solution\n');
        resid = 0;
        return;
    case 1
// This part of code is called usually many times. User should respond properly using resid()
// according the actual values in param().
        resid(1) = param(1)^2 + 3*param(1)*param(2) - param(2) + 3;
        resid(2) = 2*param(1)^2 - param(2) + 2*param(2)^2 - 3;
        resid(3) = -5*param(1) + 2*param(2) - 7;
        return;
    case 3
// Call to this function with fmode = 3 is performed exactly once at the end of the inversion.
// User can use the solution in param() according his/her wishes.
        mprintf('fcn3 call: Here is the result (exact solution is x1 = -1, x2 = +1)\n');
        mprintf('x1 = %10.4f x2 = %10.4f\n',param(1),param(2));
        mprintf('For details look into the file ANNI.log\n******************************\n');
        resid = 0;
end
endfunction
